@@ -4,9 +4,6 @@
  * Copyright (C) 2010 Alfred E. Heggestad
  * \ref GenericAudioStream
  */
-#define _DEFAULT_SOURCE 1
-#define _BSD_SOURCE 1
-#define _POSIX_C_SOURCE 199309L
 #include <string.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -131,7 +128,7 @@ struct audio {
 	bool started;                 /**< Stream is started flag          */
 	bool level_enabled;           /**< Audio level RTP ext. enabled    */
 	bool hold;                    /**< Local hold flag                 */
-	bool conference;              /**< Local conference flag           */
+	RE_ATOMIC bool conference;    /**< Local conference flag           */
 	uint8_t extmap_aulevel;       /**< ID Range 1-14 inclusive         */
 	audio_event_h *eventh;        /**< Event handler                   */
 	audio_level_h *levelh;        /**< Audio level handler             */
@@ -413,7 +410,6 @@ static void poll_aubuf_tx(struct audio *a)
 {
 	struct autx *tx = &a->tx;
 	struct auframe af;
-	int16_t *sampv = tx->sampv;
 	size_t sampc;
 	size_t sz;
 	struct le *le;
@@ -430,9 +426,10 @@ static void poll_aubuf_tx(struct audio *a)
 	ch = tx->ausrc_prm.ch;
 
 	/* timed read from audio-buffer */
-	auframe_init(&af, tx->src_fmt, sampv, sampc, srate, ch);
+	auframe_init(&af, tx->src_fmt, tx->sampv, sampc, srate, ch);
 	aubuf_read_auframe(tx->aubuf, &af);
 
+	mtx_lock(tx->mtx);
 	/* Process exactly one audio-frame in list order */
 	for (le = tx->filtl.head; le; le = le->next) {
 		struct aufilt_enc_st *st = le->data;
@@ -440,6 +437,7 @@ static void poll_aubuf_tx(struct audio *a)
 		if (st->af && st->af->ench)
 			err |= st->af->ench(st, &af);
 	}
+	mtx_unlock(tx->mtx);
 	if (err) {
 		warning("audio: aufilter encode: %m\n", err);
 	}
@@ -511,6 +509,16 @@ bool audio_txtelev_empty(const struct audio *au)
 }
 
 
+void audio_call_telev_handler(const struct audio *au, int key, bool end)
+{
+	if (!au)
+		return;
+
+	if (au->eventh)
+		au->eventh(key, end, au->arg);
+}
+
+
 /*
  * Read samples from Audio Source
  *
@@ -551,7 +559,10 @@ static void ausrc_read_handler(struct auframe *af, void *arg)
 	(void)aubuf_write_auframe(tx->aubuf, af);
 
 	mtx_lock(tx->mtx);
-	tx->aubuf_started = true;
+	if (!tx->aubuf_started &&
+	    (aubuf_cur_size(tx->aubuf) >= tx->psize))
+		tx->aubuf_started = true;
+
 	mtx_unlock(tx->mtx);
 
 	if (a->cfg.txmode != AUDIO_MODE_POLL)
@@ -819,6 +830,8 @@ int audio_alloc(struct audio **ap, struct list *streaml,
 	if (acc && acc->auplay_mod) {
 		err  = aurecv_set_module(a->aur, acc->auplay_mod);
 		err |= aurecv_set_device(a->aur, acc->auplay_dev);
+		if (err)
+			goto out;
 
 		info("audio: using account specific player: (%s,%s)\n",
 		     acc->auplay_mod, acc->auplay_dev);
@@ -985,6 +998,9 @@ static int aufilt_setup(struct audio *a, struct list *aufiltl)
 		struct aufilt_dec_st *decst = NULL;
 		void *ctx = NULL;
 
+		if (!af->enabled)
+			continue;
+
 		if (af->encupdh && update_enc) {
 			err = af->encupdh(&encst, &ctx, af, &encprm, a);
 			if (err) {
@@ -993,7 +1009,9 @@ static int aufilt_setup(struct audio *a, struct list *aufiltl)
 			}
 			else {
 				encst->af = af;
+				mtx_lock(tx->mtx);
 				list_append(&tx->filtl, &encst->le, encst);
+				mtx_unlock(tx->mtx);
 			}
 		}
 
@@ -1043,20 +1061,20 @@ static int start_source(struct autx *tx, struct audio *a, struct list *ausrcl)
 	/* Start Audio Source */
 	if (!tx->ausrc && ausrc_find(ausrcl, NULL) && !a->hold) {
 
-		struct ausrc_prm prm;
 		size_t sz;
 		size_t psize_alloc;
-
-		prm.srate      = srate_dsp;
-		prm.ch         = channels_dsp;
-		prm.ptime      = tx->ptime;
-		prm.fmt        = tx->src_fmt;
+		struct ausrc_prm prm = {
+			.srate      = srate_dsp,
+			.ch         = channels_dsp,
+			.ptime      = tx->ptime,
+			.fmt        = tx->src_fmt
+		};
 
 		tx->ausrc_prm = prm;
 
 		sz = aufmt_sample_size(tx->src_fmt);
 
-		psize_alloc = sz * calc_nsamp(prm.srate, prm.ch, prm.ptime);
+		psize_alloc = sz * au_calc_nsamp(prm.srate, prm.ch, prm.ptime);
 		tx->psize = psize_alloc;
 		tx->aubuf_maxsz = tx->psize * 30;
 
@@ -1081,7 +1099,7 @@ static int start_source(struct autx *tx, struct audio *a, struct list *ausrcl)
 		/* recalculate and resize aubuf if ausrc_alloc changes prm */
 		tx->src_fmt = prm.fmt;
 		sz = aufmt_sample_size(tx->src_fmt);
-		tx->psize = sz * calc_nsamp(prm.srate, prm.ch, prm.ptime);
+		tx->psize = sz * au_calc_nsamp(prm.srate, prm.ch, prm.ptime);
 		if (psize_alloc != tx->psize) {
 			tx->ausrc_prm = prm;
 			tx->aubuf_maxsz = tx->psize * 30;
@@ -1169,7 +1187,7 @@ int audio_update(struct audio *a)
 		sc = sdp_media_rformat(m, NULL);
 	}
 
-	if (!sc) {
+	if (!sc || !sc->data) {
 		info("audio: stream is disabled\n");
 		audio_stop(a);
 		return 0;
@@ -1224,22 +1242,6 @@ int audio_update(struct audio *a)
 }
 
 
-/**
- * This function simply calls audio_update() and kept for backward
- * compatibility
- *
- * @param a Audio object
- *
- * @return 0 if success, otherwise errorcode
- *
- * @deprecated Use audio_update() instead
- */
-int audio_start(struct audio *a)
-{
-	return audio_update(a);
-}
-
-
 /**
  * Start the audio source
  *
@@ -1575,7 +1577,7 @@ void audio_sdp_attr_decode(struct audio *a)
 
 				sz = aufmt_sample_size(tx->src_fmt);
 
-				tx->psize = sz * calc_nsamp(tx->ac->srate,
+				tx->psize = sz * au_calc_nsamp(tx->ac->srate,
 							    tx->ac->ch,
 							    ptime_tx);
 			}
@@ -1657,7 +1659,7 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
 	tx = &a->tx;
 	sztx = aufmt_sample_size(tx->src_fmt);
 
-	err  = re_hprintf(pf, "\n--- Audio stream ---\n");
+	err  = re_hprintf(pf, "%s", "\n--- Audio stream ---\n");
 
 	err |= re_hprintf(pf, " tx:   encode: %H ptime=%ums %s\n",
 			  aucodec_print, tx->ac,
@@ -1891,7 +1893,7 @@ int audio_set_conference(struct audio *au, bool conference)
 	if (!au)
 		return EINVAL;
 
-	au->conference = conference;
+	re_atomic_rlx_set(&au->conference, conference);
 
 	return 0;
 }
@@ -1906,7 +1908,7 @@ int audio_set_conference(struct audio *au, bool conference)
  */
 bool audio_is_conference(const struct audio *au)
 {
-	return au ? au->conference : false;
+	return au ? re_atomic_rlx(&au->conference) : false;
 }
 
 
