@@ -88,6 +88,10 @@ struct account {
 	char *ausrc_dev;
 	char *auplay_mod;
 	char *auplay_dev;
+	char *vidsrc_mod;
+	char *vidsrc_dev;
+	char *viddisp_mod;
+	char *viddisp_dev;
 	uint32_t autelev_pt;         /**< Payload type for telephone-events  */
 	char *extra;                 /**< Extra parameters                   */
 	char *uas_user;              /**< UAS authentication username        */
@@ -197,7 +201,6 @@ int custom_hdrs_print(struct re_printf *pf,
 
 int conf_get_csv(const struct conf *conf, const char *name,
 		 char *str1, size_t sz1, char *str2, size_t sz2);
-int conf_get_float(const struct conf *conf, const char *name, double *val);
 
 
 struct metric;
@@ -239,6 +242,7 @@ int  reg_json_api(struct odict *od, const struct reg *reg);
 int  reg_status(struct re_printf *pf, const struct reg *reg);
 int  reg_af(const struct reg *reg);
 const struct sa *reg_laddr(const struct reg *reg);
+const struct sa *reg_paddr(const struct reg *reg);
 void reg_set_custom_hdrs(struct reg *reg, const struct list *hdrs);
 
 /*
@@ -393,7 +397,6 @@ struct uag {
 	sip_msg_h *subh;               /**< Subscribe handler               */
 	ua_exit_h *exith;              /**< UA Exit handler                 */
 	bool nodial;                   /**< Prevent outgoing calls          */
-	bool dnd;                      /**< Do not Disturb flag             */
 	void *arg;                     /**< UA Exit handler argument        */
 	char *eprm;                    /**< Extra UA parameters             */
 #ifdef USE_TLS
@@ -405,6 +408,7 @@ struct uag {
 struct config_sip *uag_cfg(void);
 const char *uag_eprm(void);
 bool uag_delayed_close(void);
+sip_msg_h *uag_subh(void);
 int uag_raise(struct ua *ua, struct le *le);
 
 void u32mask_enable(uint32_t *mask, uint8_t bit, bool enable);
@@ -525,3 +529,4 @@ void rtprecv_mnat_connected_handler(const struct sa *raddr1,
 int  rtprecv_start_rtcp(struct rtp_receiver *rx, const char *cname,
 			const struct sa *peer, bool pinhole);
 bool rtprecv_running(const struct rtp_receiver *rx);
+void rtprecv_set_srate(struct rtp_receiver *rx, uint32_t srate);
