@@ -1051,8 +1051,10 @@ void stream_set_srate(struct stream *s, uint32_t srate_tx, uint32_t srate_rx)
 
 	if (srate_tx)
 		rtcp_set_srate_tx(s->rtp, srate_tx);
-	if (srate_rx)
+	if (srate_rx) {
 		rtcp_set_srate_rx(s->rtp, srate_rx);
+		rtprecv_set_srate(s->rx, srate_rx);
+	}
 }
 
 
@@ -1070,10 +1072,20 @@ void stream_flush(struct stream *s)
 
 void stream_enable_rtp_timeout(struct stream *strm, uint32_t timeout_ms)
 {
+	struct sdp_media *m;
+
 	if (!strm)
 		return;
 
-	if (!sdp_media_has_media(stream_sdpmedia(strm)))
+	m = stream_sdpmedia(strm);
+	if (!sdp_media_has_media(m))
+		return;
+
+	if (sdp_media_disabled(m))
+		return;
+
+	const struct sdp_format *sc = sdp_media_rformat(m, NULL);
+	if (!sc || !sc->data)
 		return;
 
 	strm->rxm.rtp_timeout = timeout_ms;
