@@ -93,6 +93,7 @@ struct vtx {
 	unsigned skipc;                    /**< Number of frames skipped  */
 	struct list filtl;                 /**< Filters in encoding order */
 	enum vidfmt fmt;                   /**< Outgoing pixel format     */
+	char module[128];                  /**< Source module name        */
 	char device[128];                  /**< Source device name        */
 	uint32_t ts_offset;                /**< Random timestamp offset   */
 	bool picup;                        /**< Send picture update       */
@@ -140,6 +141,7 @@ struct vrx {
 	struct vidsz size;                 /**< Incoming video resolution */
 	enum vidfmt fmt;                   /**< Incoming pixel format     */
 	enum vidorient orient;             /**< Display orientation       */
+	char module[128];                  /**< Display module name       */
 	char device[128];                  /**< Display device name       */
 	int pt_rx;                         /**< Incoming RTP payload type */
 	int frames;                        /**< Number of frames received */
@@ -390,6 +392,7 @@ static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame,
 		mtx_lock(vtx->lock_enc);
 
 		if (vtx->vc && vtx->vc->packetizeh) {
+			packet->picup = vtx->picup;
 			err = vtx->vc->packetizeh(vtx->enc, packet);
 			if (err)
 				goto out;
@@ -616,6 +619,7 @@ static int vtx_alloc(struct vtx *vtx, struct video *video)
 	/* The initial value of the timestamp SHOULD be random */
 	vtx->ts_offset = rand_u16();
 
+	str_ncpy(vtx->module, video->cfg.src_mod, sizeof(vtx->module));
 	str_ncpy(vtx->device, video->cfg.src_dev, sizeof(vtx->device));
 
 	vtx->fmt = (enum vidfmt)-1;
@@ -636,6 +640,7 @@ static int vrx_alloc(struct vrx *vrx, struct video *video)
 	vrx->pt_rx  = -1;
 	vrx->orient = VIDORIENT_PORTRAIT;
 
+	str_ncpy(vrx->module, video->cfg.disp_mod, sizeof(vrx->module));
 	str_ncpy(vrx->device, video->cfg.disp_dev, sizeof(vrx->device));
 
 	vrx->fmt = (enum vidfmt)-1;
@@ -727,12 +732,6 @@ static void update_rtp_timestamp(struct timestamp_recv *tsr, uint32_t rtp_ts)
 }
 
 
-static void vidframe_clear(struct vidframe *frame)
-{
-	frame->data[0] = NULL;
-}
-
-
 /**
  * Decode incoming RTP packets using the Video decoder
  *
@@ -749,7 +748,8 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 {
 	struct video *v = vrx->video;
 	struct vidframe *frame_filt = NULL;
-	struct vidframe frame_store, *frame = &frame_store;
+	struct vidframe frame_store = {.data[0] = NULL};
+	struct vidframe *frame = &frame_store;
 	struct viddec_packet pkt = {.mb = mb, .hdr = hdr};
 	struct le *le;
 	int err = 0;
@@ -772,14 +772,12 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 			  timestamp_calc_extended(vrx->ts_recv.num_wraps,
 						  vrx->ts_recv.last));
 
-	vidframe_clear(frame);
-
 	err = vrx->vc->dech(vrx->dec, frame, &pkt);
 	if (err) {
 
 		if (err != EPROTO) {
 			warning("video: %s decode error"
-				" (seq=%u, %u bytes): %m\n",
+				" (seq=%u, %zu bytes): %m\n",
 				vrx->vc->name, hdr->seq,
 				mbuf_get_left(mb), err);
 		}
@@ -906,7 +904,7 @@ static void rtcp_nack_handler(struct vtx *vtx, struct rtcp_msg *msg)
 {
 	uint16_t nack_pid;
 	uint16_t nack_blp;
-	uint16_t pids[NACK_BLPSZ + 1];
+	uint16_t pids[NACK_BLPSZ + 1] = {0};
 	struct le *le;
 
 	if (!msg || msg->hdr.count != RTCP_RTPFB_GNACK ||
@@ -964,13 +962,18 @@ static void rtcp_handler(struct stream *strm, struct rtcp_msg *msg, void *arg)
 
 	case RTCP_FIR:
 		mtx_lock(vtx->lock_enc);
+		debug("video: recv Full Intra Request (FIR)\n");
 		vtx->picup = true;
 		mtx_unlock(vtx->lock_enc);
 		break;
 
 	case RTCP_PSFB:
-		if (msg->hdr.count == RTCP_PSFB_PLI) {
-			debug("video: recv Picture Loss Indication (PLI)\n");
+		if (msg->hdr.count == RTCP_PSFB_PLI ||
+		    msg->hdr.count == RTCP_PSFB_FIR) {
+			const char *s = msg->hdr.count == RTCP_PSFB_PLI ?
+				"Picture Loss Indication (PLI)" :
+				"Full Intra Request (FIR)";
+			debug("video: recv %s\n", s);
 			mtx_lock(vtx->lock_enc);
 			vtx->picup = true;
 			mtx_unlock(vtx->lock_enc);
@@ -1050,6 +1053,7 @@ static int vrx_print_pipeline(struct re_printf *pf, const struct vrx *vrx)
  * @param streaml    List of streams
  * @param stream_prm Stream parameters
  * @param cfg        Global configuration
+ * @param acc        Account (optional)
  * @param sdp_sess   SDP Session
  * @param mnat       Media NAT (optional)
  * @param mnat_sess  Media NAT session (optional)
@@ -1067,6 +1071,7 @@ static int vrx_print_pipeline(struct re_printf *pf, const struct vrx *vrx)
 int video_alloc(struct video **vp, struct list *streaml,
 		const struct stream_param *stream_prm,
 		const struct config *cfg,
+		const struct account *acc,
 		struct sdp_session *sdp_sess,
 		const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		const struct menc *menc, struct menc_sess *menc_sess,
@@ -1095,6 +1100,26 @@ int video_alloc(struct video **vp, struct list *streaml,
 	if (err)
 		goto out;
 
+	if (acc && acc->vidsrc_mod) {
+		str_ncpy(v->vtx.module, acc->vidsrc_mod,
+			sizeof(v->vtx.module));
+		str_ncpy(v->vtx.device, acc->vidsrc_dev,
+			sizeof(v->vtx.device));
+
+		info("video: using account specific source: (%s,%s)\n",
+			acc->vidsrc_mod, acc->vidsrc_dev);
+	}
+
+	if (acc && acc->viddisp_mod) {
+		str_ncpy(v->vrx.module, acc->viddisp_mod,
+			sizeof(v->vrx.module));
+		str_ncpy(v->vrx.device, acc->viddisp_dev,
+			sizeof(v->vrx.device));
+
+		info("video: using account specific display: (%s,%s)\n",
+			acc->viddisp_mod, acc->viddisp_dev);
+	}
+
 	mem_destructor(v, video_destructor);
 
 	tmr_init(&v->tmr);
@@ -1208,7 +1233,7 @@ static int set_vidisp(struct vrx *vrx)
 	vrx->vidisp_prm.fullscreen = vrx->video->cfg.fullscreen;
 
 	vd = (struct vidisp *)vidisp_find(baresip_vidispl(),
-					  vrx->video->cfg.disp_mod);
+					  vrx->module);
 	if (!vd)
 		return ENOENT;
 
@@ -1334,7 +1359,7 @@ int video_start_source(struct video *v)
 
 	struct vtx *vtx = &v->vtx;
 
-	debug("video: start source\n");
+	debug("video: start source %s,%s\n", vtx->module, vtx->device);
 
 	if (vidsrc_find(baresip_vidsrcl(), NULL)) {
 		struct vidsrc *vs;
